/*
 *
 *  Copyright (C) 1996-2001, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Healthcare Information and Communication Systems
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Module:  wlistctn
 *
 *  Author:  Andrew Hewett
 *
 *  Purpose: Modality Worklist Server Central Test Node (ctn) Main Program
 *
 *  Last Update:      $Author: meichel $
 *  Update Date:      $Date: 2001/12/19 09:51:08 $
 *  Source File:      $Source: /share/dicom/cvs-depot/dcmtk/wlistctn/apps/wlistctn.cc,v $
 *  CVS/RCS Revision: $Revision: 1.33 $
 *  Status:           $State: Exp $
 *
 *  CVS/RCS Log at end of file
 *
 */

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#ifdef HAVE_STDLIB_H
#ifndef  _BCB4
/* workaround for bug in Borland C++ Builder 4 */
BEGIN_EXTERN_C
#endif
#include <stdlib.h>
#ifndef  _BCB4
END_EXTERN_C
#endif
#endif

BEGIN_EXTERN_C
#include <stdio.h>
#include <string.h>
#ifdef HAVE_STDARG_H
#include <stdarg.h>
#endif
#ifdef HAVE_LIBC_H
#include <libc.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#include <time.h>
#ifdef HAVE_SYS_WAIT_H
#include <sys/wait.h>
#endif
#include <errno.h>
#ifdef HAVE_SYS_TIME_H
#include <sys/time.h>
#endif
#ifdef HAVE_SYS_RESOURCE_H
#include <sys/resource.h>
#endif
#ifdef HAVE_SYS_PARAM_H
#include <sys/param.h>
#endif
#ifdef HAVE_SYS_ERRNO_H
#include <sys/errno.h>
#endif
END_EXTERN_C

#include "dicom.h"
#include "wlistctn.h"
#include "dimse.h"
#include "sce.h"
#include "wrklstdb.h"
#include "dcdict.h"
#include "dcdebug.h"
#include "ofconapp.h"
#include "cmdlnarg.h"
#include "dcuid.h"    /* for dcmtk version name */

#define OFFIS_CONSOLE_APPLICATION "wlistctn"

static char rcsid[] = "$dcmtk: " OFFIS_CONSOLE_APPLICATION " v"
  OFFIS_DCMTK_VERSION " " OFFIS_DCMTK_RELEASEDATE " $";

#define APPLICATIONTITLE        "WLISTCTN"

typedef struct _ctn_processslot CTN_ProcessSlot;

struct _ctn_processslot {
        DIC_NODENAME            peerName;
        DIC_AE                  callingAETitle;
        DIC_AE                  calledAETitle;
        int                     processId;
        time_t                  startTime;
        OFBool                  hasStorageAbility;
        CTN_ProcessSlot         *next;
};

typedef struct {
        int                     pcnt;
        CTN_ProcessSlot         *plist;
} CTN_ProcessTable;

static CTN_ProcessTable processTable;

const char *supportedAbstractSyntaxes[] = {
    UID_VerificationSOPClass,
    UID_FINDModalityWorklistInformationModel
};

int numberOfSupportedAbstractSyntaxes = DIM_OF(supportedAbstractSyntaxes);

/* Reasons for refusing association */
typedef enum {
        CTN_TooManyAssociations,
        CTN_CannotFork,
        CTN_BadAppContext,
        CTN_BadAEPeer,
        CTN_BadAEService,
        CTN_NoReason
} CTN_RefuseReason;

const char *baseWorklistDBPath = "/home/www/wlist";

OFCmdUnsignedInt   opt_port = 0;
OFBool             opt_refuseAssociation = OFFalse;
OFBool             opt_rejectWithoutImplementationUID = OFFalse;
OFCmdUnsignedInt   opt_sleepAfterFind = 0;
OFCmdUnsignedInt   opt_sleepDuringFind = 0;
OFCmdUnsignedInt   opt_maxPDU = ASC_DEFAULTMAXPDU;
E_TransferSyntax   opt_networkTransferSyntax = EXS_Unknown;
E_GrpLenEncoding   opt_groupLength = EGL_recalcGL;
E_EncodingType     opt_sequenceType = EET_ExplicitLength;
OFBool             opt_verbose = OFFalse;
OFBool             opt_debug = OFFalse;
OFBool             opt_failInvalidQuery = OFTrue;
#ifdef HAVE_FORK
static OFBool      opt_singleProcess = OFFalse;
#else
static OFBool      opt_singleProcess = OFTrue;
#endif
static int         opt_maxAssociations = 20;    /* default */

void errmsg(const char* msg, ...)
{
    va_list args;

    fprintf(stderr, "%s: ", OFFIS_CONSOLE_APPLICATION);
    va_start(args, msg);
    vfprintf(stderr, msg, args);
    va_end(args);
    fprintf(stderr, "\n");
}

static OFCondition waitForAssociation(T_ASC_Network *net);
static void cleanChildren();
static OFCondition negotiateAssociation(T_ASC_Association *assoc);

#define SHORTCOL 4
#define LONGCOL 21

int
main(int argc, char *argv[])
{
    T_ASC_Network *net;

    /*
    ** Don't let dcmdata remove tailing blank padding or perform other
    ** manipulations.  We want to see the real data.
    */
    dcmEnableAutomaticInputDataCorrection.set(OFFalse);
    fprintf(stdout, "NOTICE: Dcmdata auto correction disabled!!\n");

#ifdef HAVE_GUSI_H
    /* needed for Macintosh */
    GUSISetup(GUSIwithSIOUXSockets);
    GUSISetup(GUSIwithInternetSockets);
#endif

#ifdef HAVE_WINSOCK_H
    WSAData winSockData;
    /* we need at least version 1.1 */
    WORD winSockVersionNeeded = MAKEWORD( 1, 1 );
    WSAStartup(winSockVersionNeeded, &winSockData);
#endif

    char tempstr[20];
    OFConsoleApplication app(OFFIS_CONSOLE_APPLICATION , "DICOM Basic Worklist Management SCP", rcsid);
    OFCommandLine cmd;

    cmd.setParamColumn(LONGCOL+SHORTCOL+4);
    cmd.addParam("port", "tcp/ip port number to listen on");

    cmd.setOptionColumns(LONGCOL, SHORTCOL);
    cmd.addGroup("general options:", LONGCOL, SHORTCOL+2);
        cmd.addOption("--help",                      "-h",        "print this help text and exit");
        cmd.addOption("--verbose",                   "-v",        "verbose mode, print processing details");
        cmd.addOption("--debug",                     "-d",        "debug mode, print debug information");
#ifdef HAVE_FORK
        cmd.addOption("--single-process",            "-s",        "single process mode");
#endif
        OFString opt5 = "path to worklist database files\n(default: ";
        opt5 += baseWorklistDBPath;
        opt5 += ")";
        cmd.addOption("--database-path",             "-db",    1, "[p]ath: string", opt5.c_str() );

    cmd.addGroup("network options:");
        cmd.addSubGroup("preferred network transfer syntaxes:");
            cmd.addOption("--prefer-uncompr",        "+x=",       "prefer explicit VR local byte order (default)");
            cmd.addOption("--prefer-little",         "+xe",       "prefer explicit VR little endian TS");
            cmd.addOption("--prefer-big",            "+xb",       "prefer explicit VR big endian TS");
            cmd.addOption("--implicit",              "+xi",       "accept implicit VR little endian TS only");
        cmd.addSubGroup("other network options:");
            cmd.addOption("--refuse",                             "refuse association");
            cmd.addOption("--reject",                             "reject association if no implement. class UID");
            cmd.addOption("--no-fail",                            "don't fail on an invalid query");
            cmd.addOption("--sleep-after",                     1, "[s]econds: integer", "sleep s seconds after find (default: 0)");
            cmd.addOption("--sleep-during",                    1, "[s]econds: integer", "sleep s seconds during find (default: 0)");
            OFString opt3 = "set max receive pdu to n bytes (default: ";
            sprintf(tempstr, "%ld", (long)ASC_DEFAULTMAXPDU);
            opt3 += tempstr;
            opt3 += ")";
            OFString opt4 = "[n]umber of bytes: integer [";
            sprintf(tempstr, "%ld", (long)ASC_MINIMUMPDUSIZE);
            opt4 += tempstr;
            opt4 += "..";
            sprintf(tempstr, "%ld", (long)ASC_MAXIMUMPDUSIZE);
            opt4 += tempstr;
            opt4 += "]";
            cmd.addOption("--max-pdu",                "-pdu",   1,  opt4.c_str(), opt3.c_str());
            cmd.addOption("--disable-host-lookup",    "-dhl",      "disable hostname lookup");
        cmd.addSubGroup("group length encoding (when sending C-FIND response data):");
            cmd.addOption("--group-length-recalc",    "+g=",       "recalculate group lengths if present (default)");
            cmd.addOption("--group-length-create",    "+g",        "always write with group length elements");
            cmd.addOption("--group-length-remove",    "-g",        "always write without group length elements");
        cmd.addSubGroup("length encoding in sequences and items (when sending C-FIND response data):");
            cmd.addOption("--length-explicit",        "+e",        "write with explicit lengths (default)");
            cmd.addOption("--length-undefined",       "-e",        "write with undefined lengths");

    cmd.addGroup("encoding options:");
        cmd.addSubGroup("post-1993 value representations:");
            cmd.addOption("--enable-new-vr",          "+u",        "enable support for new VRs (UN/UT) (default)");
            cmd.addOption("--disable-new-vr",         "-u",        "disable support for new VRs, convert to OB");

    /* evaluate command line */
    prepareCmdLineArgs(argc, argv, OFFIS_CONSOLE_APPLICATION);
    if (app.parseCommandLine(cmd, argc, argv, OFCommandLine::ExpandWildcards))
    {
        /* check for --help first */
        if (cmd.findOption("--help")) app.printUsage();

        app.checkParam(cmd.getParamAndCheckMinMax(1, opt_port, 1, 65535));
        if (cmd.findOption("--verbose")) opt_verbose = OFTrue;
        if (cmd.findOption("--debug"))
        {
            opt_debug = OFTrue;
            DUL_Debug(OFTrue);
            DIMSE_debug(OFTrue);
            SetDebugLevel(3);
        }
#ifdef HAVE_FORK
        if (cmd.findOption("--single-process")) opt_singleProcess = OFTrue;
#endif
        if (cmd.findOption("--database-path")) app.checkValue(cmd.getValue(baseWorklistDBPath));
        cmd.beginOptionBlock();
        if (cmd.findOption("--prefer-uncompr"))  opt_networkTransferSyntax = EXS_Unknown;
        if (cmd.findOption("--prefer-little"))   opt_networkTransferSyntax = EXS_LittleEndianExplicit;
        if (cmd.findOption("--prefer-big"))      opt_networkTransferSyntax = EXS_BigEndianExplicit;
        if (cmd.findOption("--implicit"))        opt_networkTransferSyntax = EXS_LittleEndianImplicit;
        cmd.endOptionBlock();
        if (cmd.findOption("--refuse")) opt_refuseAssociation = OFTrue;
        if (cmd.findOption("--reject")) opt_rejectWithoutImplementationUID = OFTrue;
        if (cmd.findOption("--no-fail")) opt_failInvalidQuery = OFFalse;
        if (cmd.findOption("--sleep-after")) app.checkValue(cmd.getValueAndCheckMin(opt_sleepAfterFind, 0));
        if (cmd.findOption("--sleep-during")) app.checkValue(cmd.getValueAndCheckMin(opt_sleepDuringFind, 0));
        if (cmd.findOption("--max-pdu")) app.checkValue(cmd.getValueAndCheckMinMax(opt_maxPDU, ASC_MINIMUMPDUSIZE, ASC_MAXIMUMPDUSIZE));
        if (cmd.findOption("--disable-host-lookup")) dcmDisableGethostbyaddr.set(OFTrue);
        cmd.beginOptionBlock();
        if (cmd.findOption("--group-length-recalc")) opt_groupLength = EGL_recalcGL;
        if (cmd.findOption("--group-length-create")) opt_groupLength = EGL_withGL;
        if (cmd.findOption("--group-length-remove")) opt_groupLength = EGL_withoutGL;
        cmd.endOptionBlock();
        cmd.beginOptionBlock();
        if (cmd.findOption("--length-explicit")) opt_sequenceType = EET_ExplicitLength;
        if (cmd.findOption("--length-undefined")) opt_sequenceType = EET_UndefinedLength;
        cmd.endOptionBlock();
        cmd.beginOptionBlock();
        if (cmd.findOption("--enable-new-vr"))
        {
            dcmEnableUnknownVRGeneration.set(OFTrue);
            dcmEnableUnlimitedTextVRGeneration.set(OFTrue);
        }
        if (cmd.findOption("--disable-new-vr"))
        {
            dcmEnableUnknownVRGeneration.set(OFFalse);
            dcmEnableUnlimitedTextVRGeneration.set(OFFalse);
        }
        cmd.endOptionBlock();
    }

    if (access(baseWorklistDBPath, R_OK) < 0)
    {
        errmsg("cannot access base worklist db directory");
        errmsg("%s: %s", baseWorklistDBPath, strerror(errno));
        return 10;
    }

    /* make sure data dictionary is loaded */
    if (!dcmDataDict.isDictionaryLoaded())
    {
        fprintf(stderr, "Warning: no data dictionary loaded, check environment variable: %s\n", DCM_DICT_ENVIRONMENT_VARIABLE);
    }

    processTable.pcnt = 0;
    processTable.plist = NULL;

#ifdef HAVE_GETEUID
    /* if port is privileged we must be as well */
    if (opt_port < 1024)
    {
        if (geteuid() != 0)
        {
            errmsg("cannot listen on port %d, insufficient privileges", opt_port);
            return 10;
        }
    }
#endif

    /* initialize network, i.e. create an instance of T_ASC_Network*. */
    OFCondition cond = ASC_initializeNetwork(NET_ACCEPTOR, (int)opt_port, 1000, &net);
    if (cond.bad())
    {
        errmsg("Error initialising network:");
        DimseCondition::dump(cond);
        return 10;
    }

#if defined(HAVE_SETUID) && defined(HAVE_GETUID)
    /* return to normal uid so that we can't do too much damage in case
     * things go very wrong.   Only works if the program is setuid root,
     * and run by another user.  Running as root user may be
     * potentially disasterous if this program screws up badly.
     */
    setuid(getuid());
#endif

    /* loop waiting for associations */
    while (cond.good())
    {
        cond = waitForAssociation(net);

        if (!opt_singleProcess)
        {
            cleanChildren();       /* clean up any child processes */
        }
    }

    /* drop the network, i.e. free memory of T_ASC_Network* structure. This call */
    /* is the counterpart of ASC_initializeNetwork(...) which was called above. */
    cond = ASC_dropNetwork(&net);
    if (cond.bad())
    {
        errmsg("Error dropping network:");
        DimseCondition::dump(cond);
        return 10;
    }

#ifdef HAVE_WINSOCK_H
    WSACleanup();
#endif

    return 0;
}

/*
 * Process Table List Handling
 */

static void
addProcessToTable(int pid, T_ASC_Association * assoc)
{
    CTN_ProcessSlot *ps;

    if ((ps = (CTN_ProcessSlot*)malloc(sizeof(CTN_ProcessSlot))) == NULL)
    {
        errmsg("out of memory");
        return;
    }
    /* fill in info */
    ASC_getPresentationAddresses(assoc->params, ps->peerName, NULL);
    ASC_getAPTitles(assoc->params, ps->callingAETitle, ps->calledAETitle, NULL);
    ps->processId = pid;
    ps->startTime = time(NULL);
    ps->hasStorageAbility = OFFalse;
    /* add to start of list */
    processTable.pcnt++;
    ps->next = processTable.plist;
    processTable.plist = ps;
}

#if 0
/* Unused */
static CTN_ProcessSlot* procWithCalledAETitle(const char* aeTitle)
{
    CTN_ProcessSlot *ps;
    OFBool found = OFFalse;

    if (aeTitle == NULL) return NULL;

    ps = processTable.plist;
    while (ps && !found)
    {
        found = strcmp(ps->calledAETitle, aeTitle) == 0;
        if (!found) ps = ps->next;
    }
    return ps;
}

static CTN_ProcessSlot* procWithCallingAETitle(const char* aeTitle)
{
    CTN_ProcessSlot *ps;
    OFBool found = OFFalse;

    if (aeTitle == NULL) return NULL;

    ps = processTable.plist;
    while (ps && !found)
    {
        found = strcmp(ps->callingAETitle, aeTitle) == 0;
        if (!found) ps = ps->next;
    }
    return ps;
}
#endif

#if 0
/* currently unused */
static int
countProcessesForCalledAE(const char* aeTitle)
{
    int n = 0;
    CTN_ProcessSlot *ps;

    if (aeTitle == NULL) return 0;

    ps = processTable.plist;
    while (ps)
    {
        if (strcmp(ps->calledAETitle, aeTitle) == 0)
        {
            n++;
        }
        ps = ps->next;
    }
    return n;
}
#endif

#if 0
/* Unused */
static CTN_ProcessSlot* procWithPeerName(const char* peerName)
{
    CTN_ProcessSlot *ps;
    OFBool found = OFFalse;

    if (peerName == NULL) return NULL;

    ps = processTable.plist;
    while (ps && !found)
    {
        found = strcmp(ps->peerName, peerName) == 0;
        if (!found) ps = ps->next;
    }
    return ps;
}
#endif

static int
countChildProcesses()
{
    int n = 0;
    CTN_ProcessSlot *ps;

    ps = processTable.plist;
    while (ps)
    {
        n++;
        ps = ps->next;
    }
    return n;
}

static void
removeProcessFromTable(int pid)
{
    CTN_ProcessSlot *ps, *pps;
    OFBool found = OFFalse;

    pps = NULL;
    ps = processTable.plist;

    while (ps && !found)
    {
        found = (ps->processId == pid);
        if (!found)
        {
            pps = ps;
            ps = ps->next;
        }
    }

    if (!found)
    {
        errmsg("WARNING: process %d not found", pid);
        return;
    }

    if (pps == NULL)
    {
        /* first in list */
        processTable.plist = ps->next;
    }
    else
    {
        pps->next = ps->next;
    }

    free(ps);
    processTable.pcnt--;
}

#ifdef HAVE_WAITPID
static void
cleanChildren()
{
    int options;
    int stat_loc;
    int child = 1;

    options = WNOHANG;
    while (child > 0)
    {
        child = (int)(waitpid(-1, &stat_loc, options));
        if (child == 0)
        {
            /* child not yet finished so don't complain */
        }
        else if (child < 0)
        {
            if (errno == ECHILD)
            {
                /* no children so don't complain */
            }
            else
            {
                errmsg("wait for child failed: %s", strerror(errno));
            }
        }
        else if (child > 0)
        {
            if (opt_verbose)
            {
                printf("Cleaned up after child (%d)\n", child);
            }
            /* Remove Entry from Process Table */
            removeProcessFromTable(child);
        }
    }
}
#elif HAVE_WAIT3
static void
cleanChildren()
{
#if defined(__NeXT__)
    /* some systems need a union wait as argument to wait3 */
    union wait status;
#else
    int status;
#endif
    int options;
    struct rusage rusage;
    int child = 1;

    options = WNOHANG;
    while (child > 0)
    {
        child = wait3(&status, options, &rusage);
        if (child < 0)
        {
            if (errno == ECHILD)
            {
                /* no children so don't complain */
            }
            else
            {
                errmsg("wait for child failed: %s", strerror(errno));
            }
        }
        else if (child > 0)
        {
            if (opt_verbose)
            {
                printf("Cleaned up after child (%d)\n", child);
            }
            /* Remove Entry from Process Table */
            removeProcessFromTable(child);
        }
    }
}
#else
/* don't know how to wait for my children */
static void
cleanChildren()
{
    errmsg("cannot wait for child processes");
}
#endif


/*
 * Refuse an Association
 */

static OFCondition
refuseAssociation(T_ASC_Association ** assoc, CTN_RefuseReason reason)
{
    T_ASC_RejectParameters rej;

    if (opt_verbose)
    {
        printf("Refusing Association (");
        switch (reason)
        {
            case CTN_TooManyAssociations:
                printf("TooManyAssociations");
                break;
            case CTN_CannotFork:
                printf("CannotFork");
                break;
            case CTN_BadAppContext:
                printf("BadAppContext");
                break;
            case CTN_BadAEPeer:
                printf("BadAEPeer");
                break;
            case CTN_BadAEService:
                printf("BadAEService");
                break;
            case CTN_NoReason:
                printf("NoReason");
                break;
            default:
                printf("???");
                break;
        }
        printf(")\n");
    }
    switch (reason)
    {
        case CTN_TooManyAssociations:
            rej.result = ASC_RESULT_REJECTEDTRANSIENT;
            rej.source = ASC_SOURCE_SERVICEPROVIDER_PRESENTATION_RELATED;
            rej.reason = ASC_REASON_SP_PRES_LOCALLIMITEXCEEDED;
            break;
        case CTN_CannotFork:
            rej.result = ASC_RESULT_REJECTEDPERMANENT;
            rej.source = ASC_SOURCE_SERVICEPROVIDER_PRESENTATION_RELATED;
            rej.reason = ASC_REASON_SP_PRES_TEMPORARYCONGESTION;
            break;
        case CTN_BadAppContext:
            rej.result = ASC_RESULT_REJECTEDTRANSIENT;
            rej.source = ASC_SOURCE_SERVICEUSER;
            rej.reason = ASC_REASON_SU_APPCONTEXTNAMENOTSUPPORTED;
            break;
        case CTN_BadAEPeer:
            rej.result = ASC_RESULT_REJECTEDPERMANENT;
            rej.source = ASC_SOURCE_SERVICEUSER;
            rej.reason = ASC_REASON_SU_CALLINGAETITLENOTRECOGNIZED;
            break;
        case CTN_BadAEService:
            rej.result = ASC_RESULT_REJECTEDPERMANENT;
            rej.source = ASC_SOURCE_SERVICEUSER;
            rej.reason = ASC_REASON_SU_CALLEDAETITLENOTRECOGNIZED;
            break;
        case CTN_NoReason:
        default:
            rej.result = ASC_RESULT_REJECTEDPERMANENT;
            rej.source = ASC_SOURCE_SERVICEUSER;
            rej.reason = ASC_REASON_SU_NOREASON;
            break;
    }

    OFCondition cond = ASC_rejectAssociation(*assoc, &rej);
    if (cond.bad())
    {
        fprintf(stderr, "Association Reject Failed:\n");
        DimseCondition::dump(cond);
    }

    cond = ASC_dropAssociation(*assoc);
    if (cond.bad())
    {
        fprintf(stderr, "Cannot Drop Association:\n");
        DimseCondition::dump(cond);
    }
    cond = ASC_destroyAssociation(assoc);
    if (cond.bad())
    {
        fprintf(stderr, "Cannot Destroy Association:\n");
        DimseCondition::dump(cond);
    }

    return cond;
}

/*
** Return true if path is a directory
*/

static OFBool
isaDirectory(const char* path)
{
    OFBool ok = OFFalse;
#ifdef HAVE__FINDFIRST
    struct _finddata_t fileData;
    long hFile;
    if ( ((hFile = _findfirst(path, &fileData)) != -1L) &&
         (fileData.attrib & _A_SUBDIR) )
    {
        /* it is a directory */
        _findclose(hFile);
        ok = OFTrue;
    }
#else
    DIR *d = NULL;
    if ((d = opendir(path)) != NULL)
    {
        ok = OFTrue;
        closedir(d);
    }
#endif
    return ok;
}

/*
 * Wait for and negotiate incoming association.
 */

static OFCondition
waitForAssociation(T_ASC_Network * net)
{
    OFCondition cond = EC_Normal;
#ifdef HAVE_FORK
    int pid;
#endif
    T_ASC_Association *assoc;
    char buf[BUFSIZ];
    int timeout;
    OFBool go_cleanup = OFFalse;

    if (opt_singleProcess) timeout = 1000;    
    else
    {
      if (countChildProcesses() > 0)
      {
        timeout = 1;
      } else {
        timeout = 1000;
      }
    }

    if (! go_cleanup)
    {
        if (ASC_associationWaiting(net, timeout))
        {
            cond = ASC_receiveAssociation(net, &assoc, opt_maxPDU);
            if (cond.bad())
            {
                if (opt_verbose)
                {
                    errmsg("Failed to receive association:");
                    DimseCondition::dump(cond);
                }
                go_cleanup = OFTrue;
            }
        }
        else return EC_Normal;
    }

    if (! go_cleanup)
    {
        if (opt_verbose)
        {
            printf("Association Received (%s:%s -> %s)\n", assoc->params->DULparams.callingPresentationAddress, assoc->params->DULparams.callingAPTitle, assoc->params->DULparams.calledAPTitle);
        }
        if (opt_debug)
        {
            printf("Parameters:\n");
            ASC_dumpParameters(assoc->params, COUT);
        }
        if (opt_refuseAssociation)
        {
            if (opt_verbose)
            {
                printf("Refusing Association (forced via command line)\n");
            }
            cond = refuseAssociation(&assoc, CTN_NoReason);
            go_cleanup = OFTrue;
        }
    }

    if (! go_cleanup)
    {
        /* Application Context Name */
        cond = ASC_getApplicationContextName(assoc->params, buf);
        if (cond.bad() || strcmp(buf, DICOM_STDAPPLICATIONCONTEXT) != 0)
        {
            /* reject: the application context name is not supported */
            if (opt_verbose)
            {
                errmsg("Bad AppContextName: %s", buf);
            }
            cond = refuseAssociation(&assoc, CTN_BadAppContext);
            go_cleanup = OFTrue;
        }
    }

    if (! go_cleanup)
    {
        /* Implementation Class UID */
        if (opt_rejectWithoutImplementationUID && strlen(assoc->params->theirImplementationClassUID) == 0)
        {
            /* reject: no implementation Class UID provided */
            errmsg("Refusing Association: No implementation Class UID provided");
            cond = refuseAssociation(&assoc, CTN_NoReason);
            go_cleanup = OFTrue;
        }
    }

    if (! go_cleanup)
    {
        /* too many concurrent associations ?? */
        if (countChildProcesses() >= opt_maxAssociations)
        {
            cond = refuseAssociation(&assoc, CTN_TooManyAssociations);
            go_cleanup = OFTrue;
        }
    }

    if (! go_cleanup)
    {
        /* is the called AE title known (which worklist to query)? */
        {
            char dbPath[2048];
            sprintf(dbPath, "%s/%s", baseWorklistDBPath, assoc->params->DULparams.calledAPTitle);
            if (!isaDirectory(dbPath))
            {
                /* there is no subdirectory for this AE Title */
                cond = refuseAssociation(&assoc, CTN_BadAEService);
                go_cleanup = OFTrue;
            }
        }
    }

    if (! go_cleanup)
    {
        cond = negotiateAssociation(assoc);
        if (cond.bad()) go_cleanup = OFTrue;
    }

    if (! go_cleanup)
    {
        cond = ASC_acknowledgeAssociation(assoc);
        if (cond.bad())
        {
            DimseCondition::dump(cond);
            go_cleanup = OFTrue;
        }
    }

    if (! go_cleanup)
    {
        if (opt_verbose)
        {
            printf("Association Acknowledged (Max Send PDV: %lu)\n", assoc->sendPDVLength);
            if (ASC_countAcceptedPresentationContexts(assoc->params) == 0)
                printf("    (but no valid presentation contexts)\n");
            if (opt_debug)
                ASC_dumpParameters(assoc->params, COUT);
        }
        
        if (opt_singleProcess)
        {
            /* don't spawn a sub-process to handle the association */
            cond = SCE_handleAssociation(assoc);
        }
#ifdef HAVE_FORK
        else
        {
            /* spawn a sub-process to handle the association */
            pid = (int)(fork());
            if (pid < 0)
            {
                errmsg("Cannot create association sub-process: %s", strerror(errno));
                cond = refuseAssociation(&assoc, CTN_CannotFork);
                go_cleanup = OFTrue;
            }
            else if (pid > 0)
            {
                /* parent process, note process in table */
                addProcessToTable(pid, assoc);
            }
            else
            {
                /* child process, handle the association */
                cond = SCE_handleAssociation(assoc);
                /* the child process is done so exit */
                exit(0);
            }
        }
#endif
    }

    // cleanup code
    if (!opt_singleProcess)
    {
        /* the child will handle the association, we can drop it */
        cond = ASC_dropAssociation(assoc);
        if (cond.bad())
        {
            errmsg("Cannot Drop Association:");
            DimseCondition::dump(cond);
        }
        cond = ASC_destroyAssociation(&assoc);
        if (cond.bad())
        {
            errmsg("Cannot Destroy Association:");
            DimseCondition::dump(cond);
        }
    }

    return cond;
}


static OFCondition
negotiateAssociation(T_ASC_Association * assoc)
{
    DIC_AE calledAETitle;

    ASC_getAPTitles(assoc->params, NULL, calledAETitle, NULL);

    const char* transferSyntaxes[] = { NULL, NULL, NULL };
    int numTransferSyntaxes = 0;

    switch (opt_networkTransferSyntax)
    {
        case EXS_LittleEndianImplicit:
            /* we only support Little Endian Implicit */
            transferSyntaxes[0]  = UID_LittleEndianImplicitTransferSyntax;
            numTransferSyntaxes = 1;
            break;
        case EXS_LittleEndianExplicit:
            /* we prefer Little Endian Explicit */
            transferSyntaxes[0] = UID_LittleEndianExplicitTransferSyntax;
            transferSyntaxes[1] = UID_LittleEndianImplicitTransferSyntax;
            numTransferSyntaxes = 2;
            break;
        case EXS_BigEndianExplicit:
            /* we prefer Big Endian Explicit */
            transferSyntaxes[0] = UID_BigEndianExplicitTransferSyntax;
            transferSyntaxes[1] = UID_LittleEndianImplicitTransferSyntax;
            numTransferSyntaxes = 2;
            break;

        default:
            /* We prefer explicit transfer syntaxes.
             * If we are running on a Little Endian machine we prefer
             * LittleEndianExplicitTransferSyntax to BigEndianTransferSyntax.
             */
            if (gLocalByteOrder == EBO_LittleEndian)  /* defined in dcxfer.h */
            {
              transferSyntaxes[0] = UID_LittleEndianExplicitTransferSyntax;
              transferSyntaxes[1] = UID_BigEndianExplicitTransferSyntax;
            }
            else
            {
              transferSyntaxes[0] = UID_BigEndianExplicitTransferSyntax;
              transferSyntaxes[1] = UID_LittleEndianExplicitTransferSyntax;
            }
            transferSyntaxes[2] = UID_LittleEndianImplicitTransferSyntax;
            numTransferSyntaxes = 3;
            break;
    }

    /*  accept any of the supported abstract syntaxes */
    OFCondition cond = ASC_acceptContextsWithPreferredTransferSyntaxes( assoc->params, (const char**)supportedAbstractSyntaxes, numberOfSupportedAbstractSyntaxes, (const char**)transferSyntaxes, numTransferSyntaxes);   //bugfix by wilkens 01/11/08: 5th argument was passed "DIM_OF(transferSyntaxes)"; this leads to a segmentation fault under certain circumstances. Correct argument is "numTransferSyntaxes" (see also imagectn.cc)
    if (cond.bad())
    {
        errmsg("Cannot accept presentation contexts:");
        DimseCondition::dump(cond);
    }

    return cond;
}


/*
** CVS Log
** $Log: wlistctn.cc,v $
** Revision 1.33  2001/12/19 09:51:08  meichel
** Restructured code (removed gotos) to avoid "sorry, not implemented" error
**   on Sun CC 2.0.1
**
** Revision 1.32  2001/11/12 16:11:11  meichel
** Removed all ctndisp related code from wlistctn
**
** Revision 1.31  2001/11/09 16:00:53  joergr
** Renamed some of the getValue/getParam methods to avoid ambiguities reported
** by certain compilers.
**
** Revision 1.30  2001/11/08 15:45:12  wilkens
** Changed wlistctn tool so that it makes use of OFConsoleApplication and
** OFCommandLine.
** Fixed a bug in wlistctn: wrong argument passed to function
** ASC_acceptContextsWithPreferredTransferSyntaxes(...) lead to segmentation
** fault under certain circumstances.
**
** Revision 1.29  2001/10/12 12:02:29  meichel
** Adapted wlistctn to OFCondition based dcmnet module (supports strict mode).
**
** Revision 1.28  2001/06/01 15:51:44  meichel
** Updated copyright header
**
** Revision 1.27  2001/06/01 11:02:04  meichel
** Implemented global flag and command line option to disable reverse
**   DNS hostname lookup using gethostbyaddr when accepting associations.
**
** Revision 1.26  2000/12/12 17:45:52  joergr
** Added '#include <libc.h>' to avoid problems with gcc 2.5.8 on NeXTSTEP 3.x
** systems.
**
** Revision 1.25  2000/06/07 15:19:08  meichel
** Output stream now passed as mandatory parameter to ASC_dumpParameters.
**
** Revision 1.24  2000/04/14 16:40:57  meichel
** Global VR generation flags are now derived from OFGlobal and, thus,
**   safe for use in multi-thread applications.
**
** Revision 1.23  2000/03/08 16:37:45  meichel
** Updated copyright header.
**
** Revision 1.22  2000/02/29 11:59:33  meichel
** Removed support for VS value representation. This was proposed in CP 101
**   but never became part of the standard.
**
** Revision 1.21  2000/02/23 15:13:57  meichel
** Corrected macro for Borland C++ Builder 4 workaround.
**
** Revision 1.20  2000/02/02 15:18:32  meichel
** Replaced some #if statements by more robust #ifdef
**
** Revision 1.19  2000/02/01 11:00:59  meichel
** Avoiding to include <stdlib.h> as extern "C" on Borland C++ Builder 4,
**   workaround for bug in compiler header files.
**
** Revision 1.18  1999/05/03 14:14:46  joergr
** Minor code purifications to keep Sun CC 2.0.1 quiet.
**
** Revision 1.17  1999/04/26 12:59:12  meichel
** Ported Worklist CTN to Win32 environment.
**
** Revision 1.16  1998/07/24 13:30:20  meichel
** Minor changes to avoid compiler warnings (gcc 2.8.1 with additional
**   options), e.g. add copy constructors.
**
** Revision 1.15  1998/02/06 15:07:43  meichel
** Removed many minor problems (name clashes, unreached code)
**   reported by Sun CC4 with "+w" or Sun CC2.
**
** Revision 1.14  1998/01/14 14:27:27  hewett
** Modified existing -u command line option to also disable generation
** of UT and VS (previously just disabled generation of UN).
**
** Revision 1.13  1997/09/18 10:09:12  meichel
** Removed unused (and undocumented) option -XF from wlistctn.
**
** Revision 1.12  1997/09/18 08:11:06  meichel
** Many minor type conflicts (e.g. long passed as int) solved.
**
** Revision 1.11  1997/07/21 09:08:52  andreas
** - Replace all boolean types (BOOLEAN, CTNBOOLEAN, DICOM_BOOL, BOOL)
**   with one unique boolean type OFBool.
**
** Revision 1.10  1997/07/04 09:25:00  meichel
** Simplified some sizeof() constructs to avoid compiler warnings
**   on the IBM xlC compiler (AIX 3.x).
**
** Revision 1.9  1997/05/29 17:06:40  meichel
** All dcmtk applications now contain a version string
** which is displayed with the command line options ("usage" message)
** and which can be queried in the binary with the "ident" command.
**
** Revision 1.8  1997/05/22 13:35:57  hewett
** Modified the test for presence of a data dictionary to use the
** method DcmDataDictionary::isDictionaryLoaded().
**
** Revision 1.7  1997/03/27 16:11:50  hewett
** Added command line switches allowing generation of UN to
** be disabled (it is enabled by default).
**
** Revision 1.6  1996/09/27 08:54:27  hewett
** Enclosed system include files with BEGIN_EXTERN_C/END_EXTERN_C
**
** Revision 1.5  1996/09/24 15:49:54  hewett
** Added conditionaly compiled support for the Macintosh
** environment (GUSI library).
**
** Revision 1.4  1996/08/29 16:16:10  hewett
** Corrected copyright header.
**
** Revision 1.3  1996/05/31 13:50:44  hewett
** Now checks that the called AE Title corresponds to a Worklist database
** directory.  If no such directory exists then the association is rejected
** due to an unknown AE Title.
**
** Revision 1.2  1996/05/01 17:33:50  hewett
** Modified for latest Worklist DB.
**
** Revision 1.1.1.1  1996/04/26 14:16:01  hewett
** Initial Import.
**
*/
